@@ -8,8 +8,12 @@
 // Additions Copyright (c) 2011 Archaeopteryx Software, Inc. d/b/a Wingware
 // Scintilla platform layer for Qt
 
+#include <cstdio>
+
 #include "PlatQt.h"
 #include "Scintilla.h"
+#include "UniConversion.h"
+#include "DBCS.h"
 #include "FontQuality.h"
 
 #include <QApplication>
@@ -33,12 +37,8 @@
 #include <QTextLayout>
 #include <QTextLine>
 #include <QLibrary>
-#include <QElapsedTimer>
-#include <cstdio>
 
-#ifdef SCI_NAMESPACE
 namespace Scintilla {
-#endif
 
 //----------------------------------------------------------------------
 
@@ -102,28 +102,34 @@ public:
 	}
 	~FontAndCharacterSet() {
 		delete pfont;
-		pfont = 0;
+		pfont = nullptr;
 	}
 };
 
-static int FontCharacterSet(Font &f)
-{
-	return reinterpret_cast<FontAndCharacterSet *>(f.GetID())->characterSet;
+namespace {
+
+FontAndCharacterSet *AsFontAndCharacterSet(const Font &f) {
+	return reinterpret_cast<FontAndCharacterSet *>(f.GetID());
 }
 
-static QFont *FontPointer(Font &f)
+int FontCharacterSet(const Font &f)
 {
-	return reinterpret_cast<FontAndCharacterSet *>(f.GetID())->pfont;
+	return AsFontAndCharacterSet(f)->characterSet;
 }
 
-Font::Font() : fid(0) {}
+QFont *FontPointer(const Font &f)
+{
+	return AsFontAndCharacterSet(f)->pfont;
+}
 
+}
+
+Font::Font() noexcept : fid(nullptr) {}
 Font::~Font()
 {
 	delete reinterpret_cast<FontAndCharacterSet *>(fid);
-	fid = 0;
+	fid = nullptr;
 }
-
 static QFont::StyleStrategy ChooseStrategy(int eff)
 {
 	switch (eff) {
@@ -153,19 +159,30 @@ void Font::Release()
 {
 	if (fid)
 		delete reinterpret_cast<FontAndCharacterSet *>(fid);
-
-	fid = 0;
+	fid = nullptr;
 }
-
-
 SurfaceImpl::SurfaceImpl()
-: device(0), painter(0), deviceOwned(false), painterOwned(false), x(0), y(0),
-	  unicodeMode(false), codePage(0), codecName(0), codec(0)
+: device(nullptr), painter(nullptr), deviceOwned(false), painterOwned(false), x(0), y(0),
+	  unicodeMode(false), codePage(0), codecName(nullptr), codec(nullptr)
 {}
-
 SurfaceImpl::~SurfaceImpl()
 {
-	Release();
+	Clear();
+}
+
+void SurfaceImpl::Clear()
+{
+	if (painterOwned && painter) {
+		delete painter;
+	}
+
+	if (deviceOwned && device) {
+		delete device;
+	}
+	device = nullptr;
+	painter = nullptr;
+	deviceOwned = false;
+	painterOwned = false;
 }
 
 void SurfaceImpl::Init(WindowID wid)
@@ -197,18 +214,7 @@ void SurfaceImpl::InitPixMap(int width,
 
 void SurfaceImpl::Release()
 {
-	if (painterOwned && painter) {
-		delete painter;
-	}
-
-	if (deviceOwned && device) {
-		delete device;
-	}
-
-	device = 0;
-	painter = 0;
-	deviceOwned = false;
-	painterOwned = false;
+	Clear();
 }
 
 bool SurfaceImpl::Initialised()
@@ -274,20 +280,19 @@ void SurfaceImpl::LineTo(int x_, int y_)
 }
 
 void SurfaceImpl::Polygon(Point *pts,
-                          int npts,
+			  size_t npts,
                           ColourDesired fore,
                           ColourDesired back)
 {
 	PenColour(fore);
 	BrushColour(back);
 
-	QPoint *qpts = new QPoint[npts];
-	for (int i = 0; i < npts; i++) {
+	std::vector<QPoint> qpts(npts);
+	for (size_t i = 0; i < npts; i++) {
 		qpts[i] = QPoint(pts[i].x, pts[i].y);
 	}
 
-	GetPainter()->drawPolygon(qpts, npts);
-	delete [] qpts;
+	GetPainter()->drawPolygon(&qpts[0], static_cast<int>(npts));
 }
 
 void SurfaceImpl::RectangleDraw(PRectangle rc,
@@ -355,6 +360,26 @@ void SurfaceImpl::AlphaRectangle(PRectangle rc,
 	GetPainter()->drawRoundedRect(rect, radius, radius);
 }
 
+void SurfaceImpl::GradientRectangle(PRectangle rc, const std::vector<ColourStop> &stops, GradientOptions options) {
+	QRectF rect = QRectFFromPRect(rc);
+	QLinearGradient linearGradient;
+	switch (options) {
+	case GradientOptions::leftToRight:
+		linearGradient = QLinearGradient(rc.left, rc.top, rc.right, rc.top);
+		break;
+	case GradientOptions::topToBottom:
+	default:
+		linearGradient = QLinearGradient(rc.left, rc.top, rc.left, rc.bottom);
+		break;
+	}
+	linearGradient.setSpread(QGradient::RepeatSpread);
+	for (const ColourStop &stop : stops) {
+		linearGradient.setColorAt(stop.position, QColorFromColourAlpha(stop.colour));
+	}
+	QBrush brush = QBrush(linearGradient);
+	GetPainter()->fillRect(rect, brush);
+}
+
 static std::vector<unsigned char> ImageByteSwapped(int width, int height, const unsigned char *pixelsImage)
 {
 	// Input is RGBA, but Format_ARGB32 is BGRA, so swap the red bytes and blue bytes
@@ -440,19 +465,6 @@ void SurfaceImpl::SetClip(PRectangle rc)
 	GetPainter()->setClipRect(QRectFFromPRect(rc));
 }
 
-static size_t utf8LengthFromLead(unsigned char uch)
-{
-	if (uch >= (0x80 + 0x40 + 0x20 + 0x10)) {
-		return 4;
-	} else if (uch >= (0x80 + 0x40 + 0x20)) {
-		return 3;
-	} else if (uch >= (0x80)) {
-		return 2;
-	} else {
-		return 1;
-	}
-}
-
 void SurfaceImpl::MeasureWidths(Font &font,
                                 const char *s,
                                 int len,
@@ -469,13 +481,13 @@ void SurfaceImpl::MeasureWidths(Font &font,
 	if (unicodeMode) {
 		int fit = su.size();
 		int ui=0;
-		const unsigned char *us = reinterpret_cast<const unsigned char *>(s);
 		int i=0;
 		while (ui<fit) {
-			size_t lenChar = utf8LengthFromLead(us[i]);
-			int codeUnits = (lenChar < 4) ? 1 : 2;
+			const unsigned char uch = s[i];
+			const unsigned int byteCount = UTF8BytesOfLead[uch];
+			const int codeUnits = UTF16LengthFromUTF8ByteCount(byteCount);
 			qreal xPosition = tl.cursorToX(ui+codeUnits);
-			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
+			for (unsigned int bytePos=0; (bytePos<byteCount) && (i<len); bytePos++) {
 				positions[i++] = xPosition;
 			}
 			ui += codeUnits;
@@ -490,7 +502,7 @@ void SurfaceImpl::MeasureWidths(Font &font,
 		// DBCS
 		int ui = 0;
 		for (int i=0; i<len;) {
-			size_t lenChar = Platform::IsDBCSLeadByte(codePage, s[i]) ? 2 : 1;
+			size_t lenChar = DBCSIsLeadByte(codePage, s[i]) ? 2 : 1;
 			qreal xPosition = tl.cursorToX(ui+1);
 			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
 				positions[i++] = xPosition;
@@ -513,12 +525,6 @@ XYPOSITION SurfaceImpl::WidthText(Font &font, const char *s, int len)
 	return metrics.width(string);
 }
 
-XYPOSITION SurfaceImpl::WidthChar(Font &font, char ch)
-{
-	QFontMetricsF metrics(*FontPointer(font), device);
-	return metrics.width(ch);
-}
-
 XYPOSITION SurfaceImpl::Ascent(Font &font)
 {
 	QFontMetricsF metrics(*FontPointer(font), device);
@@ -540,12 +546,6 @@ XYPOSITION SurfaceImpl::InternalLeading(Font & /* font */)
 	return 0;
 }
 
-XYPOSITION SurfaceImpl::ExternalLeading(Font &font)
-{
-	QFontMetricsF metrics(*FontPointer(font), device);
-	return metrics.leading();
-}
-
 XYPOSITION SurfaceImpl::Height(Font &font)
 {
 	QFontMetricsF metrics(*FontPointer(font), device);
@@ -584,8 +584,7 @@ QPaintDevice *SurfaceImpl::GetPaintDevice()
 QPainter *SurfaceImpl::GetPainter()
 {
 	Q_ASSERT(device);
-
-	if (painter == 0) {
+	if (!painter) {
 		if (device->paintingActive()) {
 			painter = device->paintEngine()->painter();
 		} else {
@@ -610,7 +609,7 @@ Surface *Surface::Allocate(int)
 //----------------------------------------------------------------------
 
 namespace {
-QWidget *window(WindowID wid)
+QWidget *window(WindowID wid) noexcept
 {
 	return static_cast<QWidget *>(wid);
 }
@@ -622,16 +621,9 @@ void Window::Destroy()
 {
 	if (wid)
 		delete window(wid);
-
-	wid = 0;
+	wid = nullptr;
 }
-
-bool Window::HasFocus()
-{
-	return wid ? window(wid)->hasFocus() : false;
-}
-
-PRectangle Window::GetPosition()
+PRectangle Window::GetPosition() const
 {
 	// Before any size allocated pretend its 1000 wide so not scrolled
 	return wid ? PRectFromQRect(window(wid)->frameGeometry()) : PRectangle(0, 0, 1000, 1000);
@@ -643,9 +635,9 @@ void Window::SetPosition(PRectangle rc)
 		window(wid)->setGeometry(QRectFromPRect(rc));
 }
 
-void Window::SetPositionRelative(PRectangle rc, Window relativeTo)
+void Window::SetPositionRelative(PRectangle rc, const Window *relativeTo)
 {
-	QPoint oPos = window(relativeTo.wid)->mapToGlobal(QPoint(0,0));
+	QPoint oPos = window(relativeTo->wid)->mapToGlobal(QPoint(0,0));
 	int ox = oPos.x();
 	int oy = oPos.y();
 	ox += rc.left;
@@ -671,7 +663,7 @@ void Window::SetPositionRelative(PRectangle rc, Window relativeTo)
 	window(wid)->resize(sizex, sizey);
 }
 
-PRectangle Window::GetClientPosition()
+PRectangle Window::GetClientPosition() const
 {
 	// The client position is the window position
 	return GetPosition();
@@ -726,12 +718,6 @@ void Window::SetCursor(Cursor curs)
 	}
 }
 
-void Window::SetTitle(const char *s)
-{
-	if (wid)
-		window(wid)->setWindowTitle(s);
-}
-
 /* Returns rectangle of monitor pt is on, both rect and pt are in Window's
    window coordinates */
 PRectangle Window::GetMonitorRect(Point pt)
@@ -745,59 +731,58 @@ PRectangle Window::GetMonitorRect(Point pt)
 	        rectScreen.right(), rectScreen.bottom());
 }
 
-
 //----------------------------------------------------------------------
+class ListWidget : public QListWidget {
+public:
+	explicit ListWidget(QWidget *parent);
+	virtual ~ListWidget();
+
+	void setDelegate(IListBoxDelegate *lbDelegate);
+	void selectionChanged();
+
+protected:
+	void mouseReleaseEvent(QMouseEvent * event) override;
+	void mouseDoubleClickEvent(QMouseEvent *event) override;
+	QStyleOptionViewItem viewOptions() const override;
+
+private:
+	IListBoxDelegate *delegate;
+};
 
 class ListBoxImpl : public ListBox {
 public:
 	ListBoxImpl();
 	~ListBoxImpl();
 
-	virtual void SetFont(Font &font);
-	virtual void Create(Window &parent, int ctrlID, Point location,
-						int lineHeight, bool unicodeMode, int technology);
-	virtual void SetAverageCharWidth(int width);
-	virtual void SetVisibleRows(int rows);
-	virtual int GetVisibleRows() const;
-	virtual PRectangle GetDesiredRect();
-	virtual int CaretFromEdge();
-	virtual void Clear();
-	virtual void Append(char *s, int type = -1);
-	virtual int Length();
-	virtual void Select(int n);
-	virtual int GetSelection();
-	virtual int Find(const char *prefix);
-	virtual void GetValue(int n, char *value, int len);
-	virtual void RegisterImage(int type, const char *xpmData);
-	virtual void RegisterRGBAImage(int type, int width, int height,
-		const unsigned char *pixelsImage);
+	void SetFont(Font &font) override;
+	void Create(Window &parent, int ctrlID, Point location,
+						int lineHeight, bool unicodeMode_, int technology) override;
+	void SetAverageCharWidth(int width) override;
+	void SetVisibleRows(int rows) override;
+	int GetVisibleRows() const override;
+	PRectangle GetDesiredRect() override;
+	int CaretFromEdge() override;
+	void Clear() override;
+	void Append(char *s, int type = -1) override;
+	int Length() override;
+	void Select(int n) override;
+	int GetSelection() override;
+	int Find(const char *prefix) override;
+	void GetValue(int n, char *value, int len) override;
+	void RegisterImage(int type, const char *xpmData) override;
+	void RegisterRGBAImage(int type, int width, int height,
+		const unsigned char *pixelsImage) override;
 	virtual void RegisterQPixmapImage(int type, const QPixmap& pm);
-	virtual void ClearRegisteredImages();
-	virtual void SetDoubleClickAction(CallBackAction action, void *data);
-	virtual void SetList(const char *list, char separator, char typesep);
+	void ClearRegisteredImages() override;
+	void SetDelegate(IListBoxDelegate *lbDelegate) override;
+	void SetList(const char *list, char separator, char typesep) override;
+
+	ListWidget *GetWidget() const;
 private:
 	bool unicodeMode;
 	int visibleRows;
 	QMap<int,QPixmap> images;
 };
-
-class ListWidget : public QListWidget {
-public:
-	explicit ListWidget(QWidget *parent);
-	virtual ~ListWidget();
-
-	void setDoubleClickAction(CallBackAction action, void *data);
-
-protected:
-	virtual void mouseDoubleClickEvent(QMouseEvent *event);
-	virtual QStyleOptionViewItem viewOptions() const;
-
-private:
-	CallBackAction doubleClickAction;
-	void *doubleClickActionData;
-};
-
-
 ListBoxImpl::ListBoxImpl()
 : unicodeMode(false), visibleRows(5)
 {}
@@ -819,7 +804,7 @@ void ListBoxImpl::Create(Window &parent,
 #if defined(Q_OS_WIN)
 	// On Windows, Qt::ToolTip causes a crash when the list is clicked on
 	// so Qt::Tool is used.
-	list->setParent(0, Qt::Tool | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint
+	list->setParent(nullptr, Qt::Tool | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 		| Qt::WindowDoesNotAcceptFocus
 #endif
@@ -829,7 +814,7 @@ void ListBoxImpl::Create(Window &parent,
 	// keyboard stops working. Qt::ToolTip works but its only really
 	// documented for tooltips.
 	// On Linux / X this setting allows clicking on list items.
-	list->setParent(0, Qt::ToolTip | Qt::FramelessWindowHint);
+	list->setParent(nullptr, Qt::ToolTip | Qt::FramelessWindowHint);
 #endif
 	list->setAttribute(Qt::WA_ShowWithoutActivating);
 	list->setFocusPolicy(Qt::NoFocus);
@@ -850,13 +835,11 @@ void ListBoxImpl::Create(Window &parent,
 
 	wid = list;
 }
-
 void ListBoxImpl::SetFont(Font &font)
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
+	ListWidget *list = GetWidget();
 	list->setFont(*FontPointer(font));
 }
-
 void ListBoxImpl::SetAverageCharWidth(int /*width*/) {}
 
 void ListBoxImpl::SetVisibleRows(int rows)
@@ -868,11 +851,9 @@ int ListBoxImpl::GetVisibleRows() const
 {
 	return visibleRows;
 }
-
 PRectangle ListBoxImpl::GetDesiredRect()
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
-
+	ListWidget *list = GetWidget();
 	int rows = Length();
 	if (rows == 0 || rows > visibleRows) {
 		rows = visibleRows;
@@ -888,11 +869,9 @@ PRectangle ListBoxImpl::GetDesiredRect()
 
 	return PRectangle(0, 0, width, height);
 }
-
 int ListBoxImpl::CaretFromEdge()
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
-
+	ListWidget *list = GetWidget();
 	int maxIconWidth = 0;
 	foreach (QPixmap im, images) {
 		if (maxIconWidth < im.width())
@@ -910,17 +889,14 @@ int ListBoxImpl::CaretFromEdge()
 #endif
 	return maxIconWidth + (2 * list->frameWidth()) + extra;
 }
-
 void ListBoxImpl::Clear()
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
+	ListWidget *list = GetWidget();
 	list->clear();
 }
-
 void ListBoxImpl::Append(char *s, int type)
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
-
+	ListWidget *list = GetWidget();
 	QString str = unicodeMode ? QString::fromUtf8(s) : QString::fromLocal8Bit(s);
 	QIcon icon;
 	if (type >= 0) {
@@ -929,16 +905,14 @@ void ListBoxImpl::Append(char *s, int type)
 	}
 	new QListWidgetItem(icon, str, list);
 }
-
 int ListBoxImpl::Length()
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
+	ListWidget *list = GetWidget();
 	return list->count();
 }
-
 void ListBoxImpl::Select(int n)
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
+	ListWidget *list = GetWidget();
 	QModelIndex index = list->model()->index(n, 0);
 	if (index.isValid()) {
 		QRect row_rect = list->visualRect(index);
@@ -947,20 +921,18 @@ void ListBoxImpl::Select(int n)
 		}
 	}
 	list->setCurrentRow(n);
+	list->selectionChanged();
 }
-
 int ListBoxImpl::GetSelection()
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
+	ListWidget *list = GetWidget();
 	return list->currentRow();
 }
-
 int ListBoxImpl::Find(const char *prefix)
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
+	ListWidget *list = GetWidget();
 	QString sPrefix = unicodeMode ? QString::fromUtf8(prefix) : QString::fromLocal8Bit(prefix);
 	QList<QListWidgetItem *> ms = list->findItems(sPrefix, Qt::MatchStartsWith);
-
 	int result = -1;
 	if (!ms.isEmpty()) {
 		result = list->row(ms.first());
@@ -968,10 +940,9 @@ int ListBoxImpl::Find(const char *prefix)
 
 	return result;
 }
-
 void ListBoxImpl::GetValue(int n, char *value, int len)
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
+	ListWidget *list = GetWidget();
 	QListWidgetItem *item = list->item(n);
 	QString str = item->data(Qt::DisplayRole).toString();
 	QByteArray bytes = unicodeMode ? str.toUtf8() : str.toLocal8Bit();
@@ -983,9 +954,8 @@ void ListBoxImpl::GetValue(int n, char *value, int len)
 void ListBoxImpl::RegisterQPixmapImage(int type, const QPixmap& pm)
 {
 	images[type] = pm;
-
-	ListWidget *list = static_cast<ListWidget *>(wid);
-	if (list != NULL) {
+	ListWidget *list = GetWidget();
+	if (list) {
 		QSize iconSize = list->iconSize();
 		if (pm.width() > iconSize.width() || pm.height() > iconSize.height())
 			list->setIconSize(QSize(qMax(pm.width(), iconSize.width()),
@@ -1009,18 +979,15 @@ void ListBoxImpl::RegisterRGBAImage(int type, int width, int height, const unsig
 void ListBoxImpl::ClearRegisteredImages()
 {
 	images.clear();
-
-	ListWidget *list = static_cast<ListWidget *>(wid);
-	if (list != NULL)
+	ListWidget *list = GetWidget();
+	if (list)
 		list->setIconSize(QSize(0, 0));
 }
-
-void ListBoxImpl::SetDoubleClickAction(CallBackAction action, void *data)
+void ListBoxImpl::SetDelegate(IListBoxDelegate *lbDelegate)
 {
-	ListWidget *list = static_cast<ListWidget *>(wid);
-	list->setDoubleClickAction(action, data);
+	ListWidget *list = GetWidget();
+	list->setDelegate(lbDelegate);
 }
-
 void ListBoxImpl::SetList(const char *list, char separator, char typesep)
 {
 	// This method is *not* platform dependent.
@@ -1029,7 +996,7 @@ void ListBoxImpl::SetList(const char *list, char separator, char typesep)
 	size_t count = strlen(list) + 1;
 	std::vector<char> words(list, list+count);
 	char *startword = &words[0];
-	char *numword = NULL;
+	char *numword = nullptr;
 	int i = 0;
 	for (; words[i]; i++) {
 		if (words[i] == separator) {
@@ -1038,7 +1005,7 @@ void ListBoxImpl::SetList(const char *list, char separator, char typesep)
 				*numword = '\0';
 			Append(startword, numword?atoi(numword + 1):-1);
 			startword = &words[0] + i + 1;
-			numword = NULL;
+			numword = nullptr;
 		} else if (words[i] == typesep) {
 			numword = &words[0] + i;
 		}
@@ -1049,46 +1016,56 @@ void ListBoxImpl::SetList(const char *list, char separator, char typesep)
 		Append(startword, numword?atoi(numword + 1):-1);
 	}
 }
+ListWidget *ListBoxImpl::GetWidget() const
+{
+	return static_cast<ListWidget *>(wid);
+}
 
-ListBox::ListBox() {}
-
+ListBox::ListBox() noexcept {}
 ListBox::~ListBox() {}
 
 ListBox *ListBox::Allocate()
 {
 	return new ListBoxImpl();
 }
-
 ListWidget::ListWidget(QWidget *parent)
-: QListWidget(parent), doubleClickAction(0), doubleClickActionData(0)
+: QListWidget(parent), delegate(nullptr)
 {}
-
 ListWidget::~ListWidget() {}
 
-void ListWidget::setDoubleClickAction(CallBackAction action, void *data)
+void ListWidget::setDelegate(IListBoxDelegate *lbDelegate)
 {
-	doubleClickAction = action;
-	doubleClickActionData = data;
+	delegate = lbDelegate;
+}
+
+void ListWidget::selectionChanged() {
+	if (delegate) {
+		ListBoxEvent event(ListBoxEvent::EventType::selectionChange);
+		delegate->ListNotify(&event);
+	}
 }
 
 void ListWidget::mouseDoubleClickEvent(QMouseEvent * /* event */)
 {
-	if (doubleClickAction != 0) {
-		doubleClickAction(doubleClickActionData);
+	if (delegate) {
+		ListBoxEvent event(ListBoxEvent::EventType::doubleClick);
+		delegate->ListNotify(&event);
 	}
 }
 
+void ListWidget::mouseReleaseEvent(QMouseEvent * /* event */)
+{
+	selectionChanged();
+}
+
 QStyleOptionViewItem ListWidget::viewOptions() const
 {
 	QStyleOptionViewItem result = QListWidget::viewOptions();
 	result.state |= QStyle::State_Active;
 	return result;
 }
-
 //----------------------------------------------------------------------
-
-Menu::Menu() : mid(0) {}
-
+Menu::Menu() noexcept : mid(nullptr) {}
 void Menu::CreatePopUp()
 {
 	Destroy();
@@ -1101,9 +1078,8 @@ void Menu::Destroy()
 		QMenu *menu = static_cast<QMenu *>(mid);
 		delete menu;
 	}
-	mid = 0;
+	mid = nullptr;
 }
-
 void Menu::Show(Point pt, Window & /*w*/)
 {
 	QMenu *menu = static_cast<QMenu *>(mid);
@@ -1121,14 +1097,18 @@ public:
 		QString path = QString::fromUtf8(modulePath);
 		lib = new QLibrary(path);
 	}
+	// Deleted so DynamicLibraryImpl objects can not be copied
+	DynamicLibraryImpl(const DynamicLibraryImpl &) = delete;
+	DynamicLibraryImpl(DynamicLibraryImpl &&) = delete;
+	DynamicLibraryImpl &operator=(const DynamicLibraryImpl &) = delete;
+	DynamicLibraryImpl &operator=(DynamicLibraryImpl &&) = delete;
 
 	virtual ~DynamicLibraryImpl() {
 		if (lib)
 			lib->unload();
-		lib = 0;
+		lib = nullptr;
 	}
-
-	virtual Function FindFunction(const char *name) {
+	Function FindFunction(const char *name) override {
 		if (lib) {
 			// C++ standard doesn't like casts between function pointers and void pointers so use a union
 			union {
@@ -1142,14 +1122,12 @@ public:
 			fnConv.fp = lib->resolve(name);
 			return fnConv.f;
 		}
-		return NULL;
+		return nullptr;
 	}
-
-	virtual bool IsValid() {
-		return lib != NULL;
+	bool IsValid() override {
+		return lib != nullptr;
 	}
 };
-
 DynamicLibrary *DynamicLibrary::Load(const char *modulePath)
 {
 	return static_cast<DynamicLibrary *>(new DynamicLibraryImpl(modulePath));
@@ -1188,47 +1166,6 @@ unsigned int Platform::DoubleClickTime()
 	return QApplication::doubleClickInterval();
 }
 
-bool Platform::MouseButtonBounce()
-{
-	return false;
-}
-
-bool Platform::IsKeyDown(int /*key*/)
-{
-	return false;
-}
-
-long Platform::SendScintilla(WindowID /*w*/,
-                             unsigned int /*msg*/,
-                             unsigned long /*wParam*/,
-                             long /*lParam*/)
-{
-	return 0;
-}
-
-long Platform::SendScintillaPointer(WindowID /*w*/,
-                                    unsigned int /*msg*/,
-                                    unsigned long /*wParam*/,
-                                    void * /*lParam*/)
-{
-	return 0;
-}
-
-int Platform::Minimum(int a, int b)
-{
-	return qMin(a, b);
-}
-
-int Platform::Maximum(int a, int b)
-{
-	return qMax(a, b);
-}
-
-int Platform::Clamp(int val, int minVal, int maxVal)
-{
-	return qBound(minVal, val, maxVal);
-}
-
 void Platform::DebugDisplay(const char *s)
 {
 	qWarning("Scintilla: %s", s);
@@ -1263,77 +1200,4 @@ void Platform::Assert(const char *c, const char *file, int line)
 	}
 }
 
-
-bool Platform::IsDBCSLeadByte(int codePage, char ch)
-{
-	// Byte ranges found in Wikipedia articles with relevant search strings in each case
-	unsigned char uch = static_cast<unsigned char>(ch);
-	switch (codePage) {
-	case 932:
-		// Shift_jis
-		return ((uch >= 0x81) && (uch <= 0x9F)) ||
-		       ((uch >= 0xE0) && (uch <= 0xEF));
-	case 936:
-		// GBK
-		return (uch >= 0x81) && (uch <= 0xFE);
-	case 949:
-		// Korean Wansung KS C-5601-1987
-		return (uch >= 0x81) && (uch <= 0xFE);
-	case 950:
-		// Big5
-		return (uch >= 0x81) && (uch <= 0xFE);
-	case 1361:
-		// Korean Johab KS C-5601-1992
-		return
-		    ((uch >= 0x84) && (uch <= 0xD3)) ||
-		    ((uch >= 0xD8) && (uch <= 0xDE)) ||
-		    ((uch >= 0xE0) && (uch <= 0xF9));
-	}
-	return false;
 }
-
-int Platform::DBCSCharLength(int codePage, const char *s)
-{
-	if (codePage == 932 || codePage == 936 || codePage == 949 ||
-	        codePage == 950 || codePage == 1361) {
-		return IsDBCSLeadByte(codePage, s[0]) ? 2 : 1;
-	} else {
-		return 1;
-	}
-}
-
-int Platform::DBCSCharMaxLength()
-{
-	return 2;
-}
-
-
-//----------------------------------------------------------------------
-
-static QElapsedTimer timer;
-
-ElapsedTime::ElapsedTime() : bigBit(0), littleBit(0)
-{
-	if (!timer.isValid()) {
-		timer.start();
-	}
-	qint64 ns64Now = timer.nsecsElapsed();
-	bigBit = static_cast<unsigned long>(ns64Now >> 32);
-	littleBit = static_cast<unsigned long>(ns64Now & 0xFFFFFFFF);
-}
-
-double ElapsedTime::Duration(bool reset)
-{
-	qint64 ns64Now = timer.nsecsElapsed();
-	qint64 ns64Start = (static_cast<qint64>(static_cast<unsigned long>(bigBit)) << 32) + static_cast<unsigned long>(littleBit);
-	double result = ns64Now - ns64Start;
-	if (reset) {
-		bigBit = static_cast<unsigned long>(ns64Now >> 32);
-		littleBit = static_cast<unsigned long>(ns64Now & 0xFFFFFFFF);
-	}
-	return result / 1000000000.0;	// 1 billion nanoseconds in a second
-}
-
-#ifdef SCI_NAMESPACE
-}
-#endif
